@@ -8,25 +8,45 @@ The following class is available:
 
 #pylint: disable=attribute-defined-outside-init, ununsed-argument, no-else-return
 
-import re
 from typing import Dict, List, Any, Optional, Tuple, Union
 from datetime import datetime
 import logging
 import pandas as pd
+from sqlalchemy import delete
 from langchain_text_splitters import RecursiveCharacterTextSplitter
 from langchain.agents import Tool, AgentExecutor, create_openai_functions_agent
 from langchain.memory import ConversationBufferWindowMemory
 from langchain.prompts import ChatPromptTemplate, MessagesPlaceholder, HumanMessagePromptTemplate
 from langchain.schema import SystemMessage, HumanMessage, AIMessage, AgentAction, AgentFinish
 from langchain.load.dump import dumps
+from langchain.embeddings.base import Embeddings
 from langchain_core.callbacks.manager import CallbackManagerForChainRun
 from langchain_core.tools import BaseTool
 from langchain_community.chat_message_histories import SQLChatMessageHistory
 from langchain_community.vectorstores import FAISS
+from langchain_community.vectorstores.hanavector import HanaDB
 
-from sentence_transformers import CrossEncoder
-from hana_ai.agents.utilities import _check_generated_cap_for_bas, _inspect_python_code
-from hana_ai.vectorstore.embedding_service import GenAIHubEmbeddings
+try:
+    from sentence_transformers import CrossEncoder
+except ImportError:
+    class CrossEncoder:
+        """
+        Dummy CrossEncoder class for compatibility when sentence_transformers is not installed.
+        This class simulates the behavior of a cross-encoder by returning zeros for all predictions.
+        This is useful for testing purposes or when the actual model is not available.
+        """
+        def __init__(self, model_name):
+            self.model_name = model_name
+        def predict(self, pairs):
+            """
+            Simulate the prediction of similarity scores for pairs of texts.
+            """
+            # Dummy implementation, returns zeros
+            return [0.0 for _ in pairs]
+from hana_ml.algorithms.pal.utility import check_pal_function_exist
+from hana_ai.agents.utilities import _check_generated_cap_for_bas, _get_user_info, _inspect_python_code
+from hana_ai.vectorstore.embedding_service import GenAIHubEmbeddings, HANAVectorEmbeddings
+from hana_ai.vectorstore.pal_cross_encoder import PALCrossEncoder
 
 # Configure logging
 logging.basicConfig(level=logging.INFO)
@@ -74,19 +94,26 @@ class HANAMLRAGAgent:
     def __init__(self,
                  tools: List[Tool],
                  llm: Any,
-                 vectorstore_path: str = "chat_history_vectorstore",
                  memory_window: int = 10,
-                 long_term_db: str = "sqlite:///chat_history.db",
+                 long_term_db: str = None,
                  long_term_memory_limit: int = 1000,
-                 skip_large_data_threshold: int = 10000,
+                 skip_large_data_threshold: int = 100000,
                  chunk_size: int = 500,
                  chunk_overlap: int = 50,
                  forget_percentage: float = 0.1,
-                 max_iterations: int = 10,
-                 cross_encoder: CrossEncoder = CrossEncoder('cross-encoder/ms-marco-MiniLM-L-6-v2'),
-                 rerank_k: int = 10,
+                 max_iterations: int = 20,
+                 cross_encoder: CrossEncoder = None,
+                 embedding_service: Embeddings = None,
+                 rerank_candidates: int = 20,
+                 rerank_k: int = 3,
                  score_threshold: float = 0.5,
-                 verbose: bool = False):
+                 vector_store_type = "hanadb", # 'faiss' or 'hanadb'
+                 hana_vector_table: str = None,
+                 vectorstore_path: str = "chat_history_vectorstore", # FAISS vectorstore path
+                 drop_existing_hana_vector_table: bool = False,
+                 verbose: bool = False,
+                 session_id: str = "global_session",
+                 **kwargs):
         """
         Initialize the chatbot with integrated tools and memory systems.
 
@@ -96,8 +123,6 @@ class HANAMLRAGAgent:
             List of LangChain tools to be used by the agent.
         llm : Any
             Language model instance to be used for generating responses.
-        vectorstore_path : str
-            Path to store the vectorstore for long-term memory.
         memory_window : int
             Number of recent conversations to keep in short-term memory.
 
@@ -105,7 +130,7 @@ class HANAMLRAGAgent:
         long_term_db : str
             Connection string for long-term memory storage.
 
-            Defaults to "sqlite:///chat_history.db".
+            Defaults to HANA table "HANAAI_LONG_TERM_DB_{user}". If hana connection is not provided, a local SQLite database "sqlite:///chat_history_{user}.db" will be used.
         long_term_memory_limit : int
             Maximum number of long-term memory entries to retain.
 
@@ -113,7 +138,7 @@ class HANAMLRAGAgent:
         skip_large_data_threshold : int
             Skip storing texts longer than this threshold.
 
-            Defaults to 1000.
+            Defaults to 100000.
         chunk_size : int
             Text chunk size for embeddings.
 
@@ -129,23 +154,50 @@ class HANAMLRAGAgent:
         max_iterations : int
             Maximum number of iterations for agent execution.
 
-            Defaults to 10.
+            Defaults to 20.
         cross_encoder : CrossEncoder
             Cross-encoder model for reranking retrieved documents.
 
-            Defaults to a pre-trained cross-encoder model 'cross-encoder/ms-marco-MiniLM-L-6-v2'.            
+            Defaults to HANA cross-encoder model. If the HANA cross-encoder is not available, the sentence_transformer model 'cross-encoder/ms-marco-MiniLM-L-6-v2' will be used.
+        embedding_service : Embeddings
+            Embedding service for generating text embeddings.
+
+            Defaults to HANAVectorEmbeddings if a connection_contextis provided; otherwise, defaults to GenAIHubEmbeddings.
+        rerank_candidates : int
+            Number of candidate documents to retrieve for reranking.
+
+            Defaults to 20.           
         rerank_k : int
             Number of documents to retrieve for reranking.
 
-            Defaults to 10.
+            Defaults to 3.
         score_threshold : float
             Similarity score threshold for retrieval.
 
             Defaults to 0.5.
+        vector_store_type : str
+            Type of vector store to use for long-term memory. Options are 'faiss' or 'hanadb'.
+
+            Defaults to 'hanadb'.
+
+        hana_vector_table : str
+            Name of the HANA vector table to use for long-term memory.
+
+            Defaults to "HANA_AI_CHAT_HISTORY_{user}".
+        vectorstore_path : str
+            Path to store the vectorstore for long-term memory when vector_store_type is "hanadb".
+        drop_existing_hana_vector_table : bool
+            Whether to drop the existing HANA vector table before creating a new one.
+
+            Defaults to False.
         verbose : bool
             Whether to enable verbose logging.
 
             Defaults to False.
+        session_id : str
+            Session ID for long-term memory storage.
+
+            Defaults to "global_session".
         """
         self.llm = llm
         self.tools = tools
@@ -158,11 +210,38 @@ class HANAMLRAGAgent:
         self.chunk_overlap = chunk_overlap
         self.forget_percentage = forget_percentage
         self.max_iterations = max_iterations
+        self.rerank_candidates = rerank_candidates
         self.rerank_k = rerank_k
         self.score_threshold = score_threshold
         self.verbose = verbose
         self.cross_encoder = cross_encoder
-
+        self.vectorstore_type = vector_store_type
+        self.hana_connection_context = None
+        for tool in self.tools:
+            if hasattr(tool, 'connection_context'):
+                self.hana_connection_context = tool.connection_context
+                break
+        if self.cross_encoder is None:
+            if check_pal_function_exist(self.hana_connection_context, '%PAL_CROSSENCODER%', like=True):
+                self.cross_encoder = PALCrossEncoder(self.hana_connection_context)
+            else:
+                self.cross_encoder = CrossEncoder('cross-encoder/ms-marco-MiniLM-L-6-v2')
+        self.hana_vector_table = hana_vector_table
+        self.user = ''
+        if self.hana_vector_table is None:
+            if self.hana_connection_context:
+                self.user = _get_user_info(self.hana_connection_context)
+                self.hana_vector_table = f"HANA_AI_CHAT_HISTORY_{self.user}"
+        self.drop_existing_hana_vector_table = drop_existing_hana_vector_table
+        self.embedding_service = embedding_service
+        if self.embedding_service is None:
+            self.embedding_service = HANAVectorEmbeddings(self.hana_connection_context) if self.hana_connection_context else GenAIHubEmbeddings()
+        if self.long_term_db is None:
+            if self.hana_connection_context:
+                self.long_term_db = self.hana_connection_context.to_sqlalchemy()
+            else:
+                self.long_term_db = f"sqlite:///chat_history_{self.user}.db"
+        self.session_id = session_id
         # Initialize memory systems
         self._initialize_memory()
 
@@ -178,23 +257,62 @@ class HANAMLRAGAgent:
             return_messages=True
         )
         # Long-term memory storage
-        self.long_term_store = SQLChatMessageHistory(
-            connection_string=self.long_term_db,
-            session_id="global_session"
-        )
+        if isinstance(self.long_term_db, str):
+            self.long_term_store = SQLChatMessageHistory(
+                connection_string=self.long_term_db,
+                session_id=self.session_id
+            )
+        else:
+            table_name = f"HANAAI_LONG_TERM_DB_{self.user}"
+            if not self.hana_connection_context.has_table(table_name):
+                self.hana_connection_context.create_table(
+                    table_name,
+                    table_structure={
+                        "ID": "INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY",
+                        "SESSION_ID": "NVARCHAR(5000)",
+                        "MESSAGE": "NCLOB",
+                    }
+                )
+            self.long_term_store = SQLChatMessageHistory(
+                connection=self.long_term_db,
+                table_name=table_name,
+                session_id=self.session_id
+            )
+
         # Initialize RAG vectorstore
         self._initialize_vectorstore()
 
-    def _initialize_vectorstore(self):
+    def delete_message_long_term_store(self, message_id) -> None:
+        """
+        Delete a specific message by its 
+
+        Parameters
+        ----------
+        message_id : str
+            The ID of the message to delete from long-term memory.
+        """
+        long_term_store = self.long_term_store
+        try:
+            long_term_store._create_table_if_not_exists()
+
+            with long_term_store._make_sync_session() as session:
+                stmt = delete(long_term_store.sql_model_class).where(
+                    long_term_store.sql_model_class.id == message_id,
+                    getattr(long_term_store.sql_model_class, long_term_store.session_id_field_name) == long_term_store.session_id
+                )
+                session.execute(stmt)
+                session.commit()
+        except Exception as e:
+            logger.error("Failed to delete message with ID %s: %s", message_id, str(e))
+
+    def _initialize_faiss_vectorstore(self):
         """Initialize or load FAISS vectorstore for long-term memory"""
-        # Create embeddings service
-        self.embeddings = GenAIHubEmbeddings()
 
         # Try loading existing vectorstore
         try:
             self.vectorstore = FAISS.load_local(
                 self.vectorstore_path,
-                self.embeddings,
+                self.embedding_service,
                 allow_dangerous_deserialization=True
             )
             logger.info("Loaded existing vectorstore from %s", self.vectorstore_path)
@@ -202,25 +320,39 @@ class HANAMLRAGAgent:
             # Initialize new vectorstore if loading fails
             self.vectorstore = FAISS.from_texts(
                 texts=["Initialization text"],
-                embedding=self.embeddings
+                embedding=self.embedding_service
             )
             self.vectorstore.save_local(self.vectorstore_path)
             logger.info("Created new vectorstore at %s", self.vectorstore_path)
 
+    def _initialize_hanadb_vectorstore(self):
+        """Initialize or load HANA DB vectorstore for long-term memory"""
+        if not self.hana_connection_context:
+            raise ValueError("HANA connection context is required for HANA DB vectorstore")
+
+        # Initialize or load HANA vectorstore
+        if self.drop_existing_hana_vector_table:
+            self.hana_connection_context.drop_table(self.hana_vector_table)
+        try:
+            self.vectorstore = HanaDB(
+                embedding=self.embedding_service,
+                connection=self.hana_connection_context.connection,
+                table_name=self.hana_vector_table
+            )
+            logger.info("Initialized HANA DB vectorstore with table %s", self.hana_vector_table)
+        except Exception as e:
+            logger.error("Failed to initialize HANA DB vectorstore: %s", str(e))
+            raise e
+
+    def _initialize_vectorstore(self):
+        if self.vectorstore_type.lower() == "faiss":
+            self._initialize_faiss_vectorstore()
+        else:
+            self._initialize_hanadb_vectorstore()
+
     def _should_store(self, text: str) -> bool:
         """Determine if text should be stored in memory"""
-        # Skip DataFrame conversions
-        if isinstance(text, pd.DataFrame):
-            return False
-
-        # Skip large texts
-        if isinstance(text, str) and len(text) > self.skip_large_data_threshold:
-            return False
-
-        # Skip Markdown tables
-        if isinstance(text, str) and re.search(r'\|\s*.*\s*\|.*\n(\|\s*:?[-]+\s*:?\|\s*)+\n', text):
-            return False
-
+        # condition for the future expansion
         return True
 
     def _update_long_term_memory(self, user_input: str, response: Any):
@@ -251,30 +383,99 @@ class HANAMLRAGAgent:
 
         # Update vector store
         self.vectorstore.add_documents(documents)
-        self.vectorstore.save_local(self.vectorstore_path)
+        if self.vectorstore_type.lower() == "faiss":
+            self.vectorstore.save_local(self.vectorstore_path)
 
         # Clean up oldest memories if needed
         self._forget_old_memories()
 
+    def _forget_past_messages_in_hana_db(self, timestamp: str):
+        self.vectorstore.delete(filter={"timestamp": {"$lte": timestamp}})
+
     def _forget_old_memories(self):
         """Remove oldest memories when storage limit is exceeded"""
         if len(self.long_term_store.messages) > self.long_term_memory_limit:
             # Calculate number of oldest messages to remove
             num_to_remove = int(self.long_term_memory_limit * self.forget_percentage)
-
-            # Implement actual deletion logic here
-            # This is placeholder for actual database deletion
-            logger.info("Should remove %d oldest memories", num_to_remove)
-            # In practice: Delete oldest records from database
+            # 1. Delete from SQL database
+            try:
+                # Get sorted list of messages by timestamp
+                sorted_messages = sorted(
+                    self.long_term_store.messages,
+                    key=lambda msg: msg.metadata.get('timestamp', '1970-01-01')
+                )
+                # Delete oldest messages from SQL store
+                last_timestamp = '1970-01-01'
+                for msg in sorted_messages[:num_to_remove]:
+                    self.delete_message_long_term_store(msg.id)
+                    last_timestamp = msg.metadata.get('timestamp', last_timestamp)
+                logger.info("Deleted %s oldest memories from SQL store", num_to_remove)
+            except Exception as e:
+                logger.error("Error deleting from SQL store: %s", str(e))
+            # 2. Rebuild vectorstore from remaining messages in Faiss
+            if self.vectorstore_type.lower() == "faiss":
+                try:
+                    # Get remaining messages after deletion
+                    remaining_messages = self.long_term_store.messages
+                    # Skip if no messages left
+                    if not remaining_messages:
+                        self.vectorstore = FAISS.from_texts(
+                            [""],
+                            embedding=self.embedding_service
+                        )
+                        return
+                    # Recreate documents from remaining messages
+                    splitter = RecursiveCharacterTextSplitter(
+                        chunk_size=self.chunk_size,
+                        chunk_overlap=self.chunk_overlap,
+                    )
+                    documents = []
+                    for i in range(0, len(remaining_messages), 2):
+                        if i+1 >= len(remaining_messages):
+                            break  # Skip incomplete pairs
+                        user_msg = remaining_messages[i]
+                        ai_msg = remaining_messages[i+1]
+                        # Create combined document
+                        text = f"User: {user_msg.content}\nAssistant: {ai_msg.content}"
+                        metadata = {"timestamp": user_msg.metadata.get('timestamp', '')}
+                        # Split and add to documents
+                        docs = splitter.create_documents(
+                            texts=[text],
+                            metadatas=[metadata]
+                        )
+                        documents.extend(docs)
+                    # Rebuild vectorstore
+                    self.vectorstore = FAISS.from_documents(
+                        documents,
+                        self.embedding_service
+                    )
+                    self.vectorstore.save_local(self.vectorstore_path)
+                    logger.info("Rebuilt vectorstore with %s documents", len(documents))
+                except Exception as e:
+                    logger.error("Error rebuilding vectorstore: %s", str(e))
+                    # Fallback to empty vectorstore
+                    self.vectorstore = FAISS.from_texts(
+                        [""],
+                        embedding=self.embedding_service
+                    )
+            else:
+                self._forget_past_messages_in_hana_db(last_timestamp)
 
     def _retrieve_relevant_memories(self, query: str) -> List[str]:
         """Retrieve relevant memories using RAG with reranking"""
         # Retrieve candidate documents
-        candidate_docs = self.vectorstore.similarity_search_with_score(
-            query=query,
-            k=self.rerank_k,
-            score_threshold=self.score_threshold
-        )
+        if self.vectorstore_type.lower() == "faiss":
+            candidate_docs = self.vectorstore.similarity_search_with_score(
+                query=query,
+                k=self.rerank_candidates,
+                score_threshold=self.score_threshold
+            )
+        else:
+            candidate_docs = self.vectorstore.similarity_search_with_relevance_scores(
+                query=query,
+                k=self.rerank_candidates,
+                score_threshold=self.score_threshold
+            )
 
         if not candidate_docs:
             return []
@@ -293,7 +494,7 @@ class HANAMLRAGAgent:
         combined = list(zip(doc_contents, rerank_scores))
         combined.sort(key=lambda x: x[1], reverse=True)
 
-        return [content for content, _ in combined[:3]]
+        return [content for content, _ in combined[:self.rerank_k]]
 
     def _build_context(self, user_input: str) -> SystemMessage:
         # 获取短时记忆并转换格式
@@ -368,8 +569,17 @@ class HANAMLRAGAgent:
             logger.error("Unexpected error clearing SQL store: %s", str(e))
 
         # 重建空向量库（无需前置检查）
-        self.vectorstore = FAISS.from_texts([""], embedding=self.embeddings)  # 空文本占位
-        self.vectorstore.save_local(self.vectorstore_path)
+        if self.vectorstore_type.lower() == "faiss":
+            self.vectorstore = FAISS.from_texts([""], embedding=self.embedding_service)  # 空文本占位
+            self.vectorstore.save_local(self.vectorstore_path)
+        else:
+            self.hana_connection_context.drop_table(self.hana_vector_table)
+            self.vectorstore = HanaDB.from_texts(
+                texts=[""],
+                embedding=self.embedding_service,
+                connection=self.hana_connection_context.connection,
+                table_name=self.hana_vector_table
+            )
         logger.info("Long-term memory cleared and vectorstore reset.")
 
     def clear_short_term_memory(self):
@@ -403,6 +613,8 @@ class HANAMLRAGAgent:
             }]
         }
         response = self.executor.invoke(agent_input)
+
+        # 更新长期记忆
         self._update_long_term_memory(user_input, response['output'])
         return response['output']
 
